<?php

/*
Plugin Name: PreFormatted
Plugin URI: http://vapourtrails.ca/wp-preformatted
Version: 1.0
Description: Saves formatted versions of posts & comments to the database so that they are not formatted with every page view.
Author: Jerome Lavigne
Author URI: http://vapourtrails.ca
*/

/*  Copyright 2005  Jerome Lavigne  (email : jerome@vapourtrails.ca)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* ChangeLog:

25-Mar-2005:  Version1.0 (release)
			- minor filter re-arrange for comment content due to changed filters in WordPress.
			- added verbose comments to filter add/remove section (to remember what each is for & *why*)
			- added filter pf_fix_wptags to prevent wpauto from wrapping <!--more--> (and other) tags in their own
				paragraph blocks.  This breaks XHTML validation whether you use PreFormatted or not but it was
				simpler to fix here.

25-Feb-2005:  Version1.0-beta3
			- added GPL text, these comments and improved packaging.

24-Feb-2005:  Version1.0-beta2
			- added blockquote de-formatting to deautop filter
			- improved whitespace & paragraph tag de-formatting.

23-Feb-2005:  Version 1.0-beta.


TODO List:
	- Pre-format the_excerpt() and comment_excerpt()
	- Add filtering for other database values that pass through wptexturize()
*/

function detexturize($text) {
	// replaces html entities added by wptexturize() with their plain text equivalent
	// sections copied liberally from wptexturize()

	// *** this function expects that the content has already been processed with htmlspecialchars() !
	//      this is normally the case for the content_edit_pre hook
	
	$output = '';
	// Capture tags and everything inside them
	$textarr = preg_split("/(&lt;.*&gt;)/Us", $text, -1, PREG_SPLIT_DELIM_CAPTURE);
	//die('<pre>'.var_export($textarr, true));
	$stop = count($textarr); $next = true; // loop stuff
	for ($i = 0; $i < $stop; $i++) {
		$curl = $textarr[$i];

		if (isset($curl{0}) && (substr($curl, 0, 4) != "&lt;") && $next) { // If it's not a tag
		
			$curl = str_replace('&amp;#8212;', '---', $curl);	// em dash
			$curl = str_replace('&amp;#8211;', '--', $curl);	// en dash
			$curl = str_replace('&amp;#8230;', '...', $curl);	// to be continued
			$curl = str_replace('&amp;#8217;', "'", $curl);	// apostrophe or closing single quote
			$curl = str_replace('&amp;#8216;', "'", $curl);	// opening single quote
			$curl = str_replace('&amp;#8243;', '"', $curl);		// inches?
			$curl = str_replace('&amp;#8242;', "'", $curl);	// feet?
			
			$curl = preg_replace('/(\s|\A)&amp;#8220;(?!\s)/', '$1"$2', $curl);	// opening double quote
			$curl = str_replace('&amp;#8220;', '``', $curl);	// alternate opening double quote
			$curl = preg_replace('/&amp;#8221;(\s|\S|\Z)/', '"$1', $curl);	//closing double quote
			$curl = str_replace('&amp;#8221;', "''", $curl);	// alternate closing double quote
			
			$curl = str_replace('&amp;#8482;', '(TM)', $curl);	// trademark symbol
			$curl = str_replace('&amp;#169;', '(C)', $curl);	// copyright symbol
			$curl = str_replace('&amp;#174;', '(R)', $curl);	// registered symbol
			$curl = str_replace('&amp;#215;', 'x', $curl);	// multipication symbol?
		
		} elseif (strstr($curl, '&lt;code') || strstr($curl, '&lt;pre') || strstr($curl, '&lt;kbd' || strstr($curl, '&lt;style') || strstr($curl, '&lt;script'))) {
			// strstr is fast
			$next = false;
		} else {
			$next = true;
		}
		
		// put all ampersands back (wptexturize replaces these in html tags & code segments too)
		$curl = str_replace('&amp;#038;', '&amp;', $curl);
		$output .= $curl;
	}
	return $output;
}


function deautop($text) {

	//special case for blockquotes - wpautop moves <p> tags into block so we'll move them back outside
	$text = preg_replace('!&lt;blockquote(.*?)&gt;&lt;p&gt;!i', "&lt;p&gt;&lt;blockquote$1&gt;", $text);
	$text = str_replace('&lt;/p&gt;&lt;/blockquote&gt;', '&lt;/blockquote&gt;&lt;/p&gt;', $text);

	$text = preg_replace('!\A\s*&lt;p&gt;!', '', $text);	// remove opening <p> tag
	$text = preg_replace('!&lt;/p&gt;\s*\Z!', "\n", $text);	// remove closing </p> tag

	$text = preg_replace('!&lt;/p&gt;\s*?(\n)*\s*!', "\n$1", $text);	// separate paragraphs with 2 newlines
	$text = preg_replace('!(\n)*\s*&lt;p&gt;!', "$1\n", $text);	// separate paragraphs with 2 newlines

	$text = preg_replace('!&lt;br /&gt;\s*!', "\n", $text);			// replace line breaks with 1 newline

	return ($text);
}


function pf_compatible_content($text) {

	// don't do anything with preformatted content
	if (substr($text, 0, 4) == "\t<p>")
		return ($text);

	// else apply filters as usual
	$output = wptexturize($text);
	$output = convert_chars($output);
	$output = wpautop($output);
	return ($output);
}

function pf_compatible_comment($text) {

	// don't do anything with preformatted content
	if (substr($text, 0, 4) == "\t<p>")
		return ($text);
		
	// else apply filters as usual
	$output = wptexturize($text);
	$output = convert_chars($output);
	$output = make_clickable($output);
	$output = wpautop($output);
	return ($output);
}

function pf_fix_wptags($text) {
	/* quick fix for a common WP problem made worse by this plugin
		we attempt to move <!--more--> and <!--nextpage--> tags outside of their own <p> blocks
		can't do much with ones embedded in other paragraphs without breaking more stuff */
	$wptags = array('more', 'nextpage', 'noteaser');	//are there more?
	foreach($wptags as $tag)
		$text = preg_replace('%<p>\s*(<!--' . $tag . '-->)\s*</p>%', '$1', $text);
	return($text);
}

/***** POST FORMATTING *****/

/* 'the_content' applied to post content prior to rendering 
	WP's default filters include wptexturize, convert_smilies, convert_chars, wpautop
	*/
	remove_filter('the_content', 'wptexturize');
	remove_filter('the_content', 'convert_chars');
	remove_filter('the_content', 'wpautop');
	//remove_filter('the_content', 'convert_smilies');		this version doesn't bother with smilies

	add_filter('the_content',  'pf_compatible_content');	//compatibility with unformatted posts

/* 'content_save_pre' applied to post content when saving post from admin pages or XML-RPC
	WP's default filters include balanceTags(50)
	Need to strip & add slashes here since text comes into filter slashed and then goes direct into database
	*/
	add_filter('content_save_pre', 'stripslashes', 0);
	add_filter('content_save_pre', 'wptexturize');
	add_filter('content_save_pre', 'convert_chars');
	add_filter('content_save_pre', 'wpautop');
	add_filter('content_save_pre', 'pf_fix_wptags');
	add_filter('content_save_pre', 'addslashes', 100);

/* 'content_edit_pre' applied to post content prior to display in editing screen
	WP has no filters for this
	*/
	add_filter('content_edit_pre', 'detexturize');
	add_filter('content_edit_pre', 'deautop');


/***** COMMENT FORMATTING *****/

/* 'comment_text' applied to comment prior to rendering 
	WP's default filters include wptexturize, convert_chars, make_clickable, wpautop(30), convert_smilies(20)
	*/
	remove_filter('comment_text', 'wptexturize');
	remove_filter('comment_text', 'convert_chars');
	remove_filter('comment_text', 'make_clickable');
	remove_filter('comment_text', 'wpautop');
	//remove_filter('comment_text', 'convert_smilies');	this version doesn't bother with smilies
	
	add_filter('comment_text', 'pf_compatible_comment');	//compatibility with unformatted comments


/* 'pre_comment_content' applied to comment content prior to saving new comment into database
	WP's default filters include strip_slashes(1), wp_filter_kses, wp_rel_nofollow(15), balanceTags(30), addslashes(50)
	*/
	add_filter('pre_comment_content', 'wptexturize');
	add_filter('pre_comment_content', 'convert_chars');
	add_filter('pre_comment_content', 'make_clickable');
	add_filter('pre_comment_content', 'wpautop');

/* 'comment_save_pre' applied to comment content when re-saving comment from admin pages
	WP's default filters include balanceTags(30)
	Need to strip & add slashes here since text comes into filter slashed and then goes direct into database
	*/
	add_filter('comment_save_pre', 'stripslashes', 0);
	add_filter('comment_save_pre', 'wptexturize');
	add_filter('comment_save_pre', 'convert_chars');
	add_filter('comment_save_pre', 'make_clickable');
	add_filter('comment_save_pre', 'wpautop');
	add_filter('comment_save_pre', 'addslashes', 100);

/* 'comment_edit_pre' applied to comment content prior to display in editing screen
	WP has no filters for this
	*/
	add_filter('comment_edit_pre', 'detexturize');
	add_filter('comment_edit_pre', 'deautop');



?>